import urllib.parse
from datetime import datetime, timezone
from typing import TypedDict

import lol_id_tools as lit
from lol_dto.classes.game import LolGame, LolGameTeam, LolGamePlayer, LolGameTeamEndOfGameStats

from leaguepedia_parser.transmuters.game_players import LeaguepediaPlayerIdentifier


class LeaguepediaGameIdentifier(TypedDict):
    scoreboardIdWiki: str
    uniqueGame: str
    matchHistoryUrl: str
    overviewPage: str


def transmute_game(source_dict: dict) -> LolGame:
    """Transforms a ScoreboardGames row into a LolGame

    Some fields like team gold and kills are not present. Get_game_details should be used for that.
    """
    game = LolGame(
        sources={
            "leaguepedia": LeaguepediaGameIdentifier(
                scoreboardIdWiki=source_dict["ScoreboardID Wiki"],
                uniqueGame=source_dict["UniqueGame"],
                matchHistoryUrl=source_dict["MatchHistory"],
                overviewPage=source_dict["OverviewPage"],
            )
        },
        tournament=source_dict["Tournament"],
        start=datetime.fromisoformat(source_dict["DateTime UTC"])
        .replace(tzinfo=timezone.utc)
        .isoformat(timespec="seconds"),
        gameInSeries=int(source_dict["Gamename"].replace("Game ", "")),
        patch=source_dict["Patch"],
        duration=int(float(source_dict["Gamelength Number"] or 0) * 60),
        vod=source_dict["VOD"],
        winner="BLUE" if source_dict["Winner"] == "1" else "RED",
        teams={
            side: LolGameTeam(
                name=source_dict[f"Team{i}"],
                players=[
                    LolGamePlayer(
                        uniqueIdentifiers={
                            "leaguepedia": LeaguepediaPlayerIdentifier(
                                gameName=source_dict[f"Team{i}Players"].split(",")[idx]
                            )
                        },
                        championId=lit.get_id(champion_name, object_type="champion"),
                        championName=champion_name,
                    )
                    for idx, champion_name in enumerate(source_dict[f"Team{i}Picks"].split(","))
                ],
                bansNames=source_dict[f"Team{i}Bans"].split(","),
                bans=[lit.get_id(champion) for champion in source_dict[f"Team{i}Bans"].split(",")],
                endOfGameStats=LolGameTeamEndOfGameStats(
                    towerKills=int(source_dict[f"Team{i}Towers"] or 0),
                    dragonKills=int(source_dict[f"Team{i}Dragons"] or 0),
                    riftHeraldKills=int(source_dict[f"Team{i}RiftHeralds"] or 0),
                    baronKills=int(source_dict[f"Team{i}Barons"] or 0),
                ),
            )
            for side, i in [("BLUE", 1), ("RED", 2)]
        },
    )

    # For Riot API games, I directly parse the URL for the game to have its actual identifiers.
    if "leagueoflegends.com" in source_dict["MatchHistory"]:
        parsed_url = urllib.parse.urlparse(urllib.parse.urlparse(source_dict["MatchHistory"]).fragment)

        query = urllib.parse.parse_qs(parsed_url.query)
        platform_id, game_id = parsed_url.path.split("/")[1:]
        game_hash = query["gameHash"][0]

        game["sources"]["riotLolApi"] = {"gameId": int(game_id), "platformId": platform_id, "gameHash": game_hash}

    return game



from typing import List
from lol_dto.classes.game.lol_game import LolPickBan
import lol_id_tools as lit

# Ordered in the draft order for pro play as of June 2020
picks_bans_fields = [
    "Team1Ban1",
    "Team2Ban1",
    "Team1Ban2",
    "Team2Ban2",
    "Team1Ban3",
    "Team2Ban3",
    "Team1Pick1",
    "Team2Pick1",
    "Team2Pick2",
    "Team1Pick2",
    "Team1Pick3",
    "Team2Pick3",
    "Team2Ban4",
    "Team1Ban4",
    "Team2Ban5",
    "Team1Ban5",
    "Team2Pick4",
    "Team1Pick4",
    "Team1Pick5",
    "Team2Pick5",
]


def transmute_picks_bans(input_dict) -> List[LolPickBan]:
    pb_list = []

    for field in picks_bans_fields:
        pb_list.append(
            LolPickBan(
                championName=input_dict[field],
                championId=lit.get_id(input_dict[field], object_type="champion"),
                isBan="Ban" in field,
                team="BLUE" if "Team1" in field else "RED",
            )
        )

    return pb_list



from typing import List, TypedDict
from lol_dto.classes.game import LolGame
import lol_id_tools as lit


role_translation = {"1": "TOP", "2": "JGL", "3": "MID", "4": "BOT", "5": "SUP"}


class LeaguepediaPlayerIdentifier(TypedDict, total=False):
    gameName: str  # Defined here because it’s a leaguepedia-specific field
    name: str  # Current player name
    irlName: str  # IRL name of the player
    country: str  # Country of origin
    birthday: str  # YYYY-MM-DD
    pageId: int  # Page ID on leaguepedia


def add_players(game: LolGame, players: List[dict], add_page_id: bool = False) -> LolGame:
    """Adds additional player information from ScoreboardPlayers.
    """

    for team_side in game["teams"]:
        team_side_leaguepedia = "1" if team_side == "BLUE" else "2"

        for idx, game_player in enumerate(game["teams"][team_side]["players"]):
            try:
                # We get the player object from the Leaguepedia players list
                player_latest_data = next(
                    p
                    for p in players
                    if p["Side"] == team_side_leaguepedia
                    and lit.get_id(p["Champion"], object_type="champion") == game_player["championId"]
                )

                game_player["role"] = role_translation[player_latest_data["gameRoleNumber"]]

                unique_identifiers = LeaguepediaPlayerIdentifier(
                    name=player_latest_data.get("currentGameName"),
                    irlName=player_latest_data.get("irlName"),
                    country=player_latest_data.get("Country"),
                    birthday=player_latest_data.get("Birthdate"),
                )

                if add_page_id:
                    unique_identifiers["pageId"] = int(player_latest_data["pageId"])

                game_player["uniqueIdentifiers"] = {"leaguepedia": unique_identifiers}

            except StopIteration:
                # Since we cannot get the role properly, we try to infer it
                game_player["role"] = list(role_translation.values())[idx]

    return game



import setuptools
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="leaguepedia_parser",
    version="1.0a10",
    packages=[
        "leaguepedia_parser",
        "leaguepedia_parser/parsers",
        "leaguepedia_parser/transmuters",
        "leaguepedia_parser/site",
    ],
    url="https://github.com/mrtolkien/leaguepedia_parser",
    license="MIT",
    author='Gary "Tolki" Mialaret',
    install_requires=["mwclient", "lol_dto >= 0.1a4", "lol_id_tools >= 1.4.0"],
    author_email="gary.mialaret+pypi@gmail.com",
    description="A parser for Leaguepedia.",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from concurrent.futures.thread import ThreadPoolExecutor
from typing import List, Optional

from lol_dto.classes.game import LolGame
from lol_dto.classes.game.lol_game import LolPickBan

from leaguepedia_parser.site.leaguepedia import leaguepedia
from leaguepedia_parser.transmuters.game import transmute_game
from leaguepedia_parser.transmuters.game_players import add_players
from leaguepedia_parser.transmuters.picks_bans import picks_bans_fields, transmute_picks_bans
from leaguepedia_parser.transmuters.tournament import transmute_tournament, tournaments_fields, LeaguepediaTournament


def get_regions() -> List[str]:
    """Returns a list of all regions that appear in the Tournaments table.

    Returns:
        The list of all region names, simply strings.
    """
    regions_dicts_list = leaguepedia.query(tables="Tournaments", fields="Region", group_by="Region")

    return [row["Region"] for row in regions_dicts_list]


def get_tournaments(
    region: str = None, year: int = None, tournament_level: str = "Primary", is_playoffs: bool = None, **kwargs,
) -> List[LeaguepediaTournament]:
    """Returns a list of tournaments.

    Typical usage example:
        get_tournaments('China', 2020)

    Args:
        region: Recommended to get it from get_tournament_regions().
        year: Year to filter on. Defaults to None.
        tournament_level: Primary, Secondary, Major, Secondary, Showmatch. Defaults to Primary.
        is_playoffs: Can be used to filter between playoffs and regular season tournaments.

    Returns:
        A list of tournaments dictionaries.
    """
    # We need to cast is_playoffs as an integer for the cargoquery
    if is_playoffs is not None:
        is_playoffs = 1 if is_playoffs else 0

    # This generates the WHERE part of the cargoquery
    where = " AND ".join(
        [
            # One constraint per variable
            f"Tournaments.{field_name}='{value}'"
            for field_name, value in [
                ("Region", region),
                ("Year", year),
                ("TournamentLevel", tournament_level),
                ("IsPlayoffs", is_playoffs),
            ]
            # We don’t filter on variables that are None
            if value is not None
        ]
    )

    result = leaguepedia.query(
        tables="Tournaments, Leagues",
        join_on="Tournaments.League = Leagues.League",
        fields=f"Leagues.League_Short, {', '.join(f'Tournaments.{field}' for field in tournaments_fields)}",
        where=where,
        **kwargs,
    )

    return [transmute_tournament(tournament) for tournament in result]


def get_games(tournament_overview_page=None, **kwargs) -> List[LolGame]:
    """Returns the list of games played in a tournament.

    Returns basic information about all games played in a tournament.

    Args:
        tournament_overview_page: tournament overview page, acquired from get_tournaments().

    Returns:
        A list of LolGame with basic game information.
    """

    game_fields = {
        "Tournament",
        "Team1",
        "Team2",
        "Winner",
        "Gamelength_Number",
        "DateTime_UTC",
        "Team1Score",
        "Team2Score",
        "Team1Bans",
        "Team2Bans",
        "Team1Picks",
        "Team2Picks",
        "Team1Players",
        "Team2Players",
        "Team1Dragons",
        "Team2Dragons",
        "Team1Barons",
        "Team2Barons",
        "Team1Towers",
        "Team2Towers",
        "Team1RiftHeralds",
        "Team2RiftHeralds",
        "Team1Inhibitors",
        "Team2Inhibitors",
        "Patch",
        "MatchHistory",
        "VOD",
        "Gamename",
        "OverviewPage",
        "ScoreboardID_Wiki",
        "UniqueGame",
    }

    games = leaguepedia.query(
        tables="ScoreboardGames",
        fields=", ".join(game_fields),
        where=f"ScoreboardGames.OverviewPage ='{tournament_overview_page}'",
        order_by="ScoreboardGames.DateTime_UTC",
        **kwargs,
    )

    return [transmute_game(game) for game in games]


def get_game_details(game: LolGame, add_page_id=False) -> LolGame:
    # TODO Add more scoreboard information in this step
    """Gets most game information available on Leaguepedia.

    Args:
        game: A LolGame with Leaguepedia IDs in its 'sources' dict.
        add_page_id: whether or not to link the player page ID to their object. Mostly for debugging.

    Returns:
        The LolGame with all information available on Leaguepedia.
    """
    try:
        assert "scoreboardIdWiki" in game["sources"]["leaguepedia"]
        assert "uniqueGame" in game["sources"]["leaguepedia"]
    except AssertionError:
        raise ValueError(f"Leaguepedia Identifiers not present in the input object.")

    with ThreadPoolExecutor() as executor:
        picks_bans_future = executor.submit(_get_picks_bans, game)
        game_future = executor.submit(_add_game_players, game, add_page_id)

    game = game_future.result()
    game["picksBans"] = picks_bans_future.result()

    return game


def _get_picks_bans(game: LolGame) -> Optional[List[LolPickBan]]:
    """Returns the picks and bans for the game.
    """
    # Double join as required by Leaguepedia
    picks_bans = leaguepedia.query(
        tables="PicksAndBansS7, MatchScheduleGame, ScoreboardGames",
        join_on="PicksAndBansS7.GameID_Wiki = MatchScheduleGame.GameID_Wiki, "
        "MatchScheduleGame.ScoreboardID_Wiki = ScoreboardGames.ScoreboardID_Wiki",
        fields=", ".join(picks_bans_fields),
        where=f"ScoreboardGames.ScoreboardID_Wiki = '{game['sources']['leaguepedia']['scoreboardIdWiki']}'",
    )

    if not picks_bans:
        return None

    return transmute_picks_bans(picks_bans[0])


def _add_game_players(game: LolGame, add_page_id: bool) -> LolGame:
    """Joins on PlayersRedirect to get all players information.
    """

    game_players_fields = {
        "ScoreboardPlayers.Name=gameName",
        "ScoreboardPlayers.Role_Number=gameRoleNumber",
        "ScoreboardPlayers.Champion",
        "ScoreboardPlayers.Side",
        "Players.Name=irlName",
        "Players.Country",
        "Players.Birthdate",
        "Players.ID=currentGameName",
        # "Players.Image",
        # "Players.Team=currentTeam",
        # "Players.Role=currentRole",
        # "Players.SoloqueueIds",
    }

    players = leaguepedia.query(
        tables=f"ScoreboardGames, ScoreboardPlayers, PlayerRedirects, Players, _pageData = PD",
        join_on="ScoreboardGames.UniqueGame = ScoreboardPlayers.UniqueGame, "
        "ScoreboardPlayers.Link = PlayerRedirects.AllName, "
        "PlayerRedirects._pageName = Players._pageName, "
        "Players._pageName = PD._pageName",
        fields=", ".join(game_players_fields) + ", PD._pageID=pageId",
        where=f"ScoreboardGames.UniqueGame = '{game['sources']['leaguepedia']['uniqueGame']}'AND PD._isRedirect = 0",
    )

    return add_players(game, players, add_page_id=add_page_id)
